defmodule Mix.Tasks.Deli.Clean do
  use Mix.Task
  import Deli.Shell
  alias Deli.Config

  @moduledoc """
  This task cleans autogenerated config files

      $ mix deli.clean
  """

  @shortdoc "Cleans autogenerated config files"

  @clean_paths ~w(
    .deliver/config
    .deli/Dockerfile
    .deli/docker-compose.yml
  )

  @impl true
  def run(_args) do
    _ = ensure_all_started(:deli)
    releases_path = expand_path(".deli/releases")
    cmd(:rm, ["-rf", releases_path], [0, 1])
    Enum.each(@clean_paths, &remove_if_autogenerated/1)
  end

  defp remove_if_autogenerated(path) do
    file_handler = Config.__file_handler__()
    path = expand_path(path)

    if file_exists?(path) do
      if file_handler.read!(path) =~ ~r/autogenerated by deli/ do
        cmd(:rm, [path])
      end
    end
  end
end
